@@ -1,20 +1,9 @@
-from __future__ import absolute_import
-
 from django.conf import settings
 from django.core import exceptions
 from django.utils import importlib
 
 from datastream import Datastream
 
-try:
-    from django.db.models.constants import LOOKUP_SEP
-except ImportError:
-    # To support Django 1.4 we move to location where Django 1.5+ has constants
-    import sys
-    from django.db.models.sql import constants
-    import django.db.models
-    django.db.models.constants = constants
-    sys.modules['django.db.models.constants'] = django.db.models.constants
 
 datastream = None
 
@@ -28,12 +17,12 @@ def init_datastream(datastream_backend, datastream_backend_settings):
 
         try:
             mod = importlib.import_module(module)
-        except ImportError, e:
-            raise exceptions.ImproperlyConfigured('Error importing datastream backend %s: "%s"' % (module, e))
+        except ImportError, exception:
+            raise exceptions.ImproperlyConfigured("Error importing datastream backend %s: %s" % (module, exception))
         try:
             cls = getattr(mod, attr)
         except AttributeError:
-            raise exceptions.ImproperlyConfigured('Module "%s" does not define a "%s" datastream backend' % (module, attr))
+            raise exceptions.ImproperlyConfigured("Module '%s' does not define a '%s' datastream backend" % (module, attr))
 
         backend = cls(**datastream_backend_settings)
 
