@@ -1,4 +1,5 @@
 import datetime
+import itertools
 import optparse
 import random
 import re
@@ -13,8 +14,8 @@ from django_datastream import datastream
 
 re_float = r'[-+]?[0-9]*\.?[0-9]+'
 re_int = r'[-+]?\d+'
-check_types = re.compile(r'^(int(\(%s,%s\))?|float(\(%s,%s\))?|enum(\((\w|,)+\))?|,)*$' % (re_int, re_int, re_float, re_float))
-split_types = re.compile(r'(int|float|enum)(?:\(([^)]+)\))?')
+check_types = re.compile(r'^(int(\(%s,%s\))?|float(\(%s,%s\))?|enum(\((\w|,)+\))?|graph(\(%s,%s\))?|,)*$' % (re_int, re_int, re_float, re_float, re_int, re_int))
+split_types = re.compile(r'(int|float|enum|graph)(?:\(([^)]+)\))?')
 
 DEMO_TYPE = 'int(0,10),float(-2,2),float(0,100)'
 DEMO_SPAN = '2d'
@@ -22,10 +23,52 @@ DEMO_SPAN = '2d'
 DEFAULT_NSTREAMS = 3
 DEFAULT_INTERVAL = 5 # seconds
 
+
+def random_graph(number_of_nodes, number_of_edges):
+    # Produces a graph picked randomly out of the set of all graphs
+    # with number_of_nodes nodes and number_of_edges edges. Based on
+    # NetworkX gnm_random_graph.
+
+    nodes = range(number_of_nodes)
+    graph = {
+        'v': [{'i': i} for i in nodes],
+        'e': [],
+    }
+
+    if number_of_nodes == 1:
+        return graph
+
+    max_edges = number_of_nodes * (number_of_nodes - 1)
+    if number_of_edges >= max_edges:
+        # Complete graph.
+        for f, t in itertools.permutations(nodes, 2):
+            graph['e'].append({
+                'f': f,
+                't': t,
+            })
+        return graph
+
+    edges = set()
+    while len(edges) < number_of_edges:
+        f = random.choice(nodes)
+        t = random.choice(nodes)
+        if f == t or (f, t) in edges:
+            continue
+        else:
+            graph['e'].append({
+                'f': f,
+                't': t,
+            })
+            edges.add((f, t))
+
+    return graph
+
+
 TYPES = {
     'int': (int, random.randint, '0,100'),
     'float': (float, random.uniform, '0,100'),
     'enum': (str, lambda *x: random.choice(x), 'a,b,c'),
+    'graph': (int, random_graph, '4,6'),
 }
 
 
@@ -41,7 +84,7 @@ class Command(base.BaseCommand):
         ),
         optparse.make_option(
             '--types', '-t', action='store', type='string', dest='types',
-            help="Stream types given as comma-separated values of 'int', 'float', or 'enum' (default: empty string). Range can be specified in brackets.",
+            help="Stream types given as comma-separated values of 'int', 'float', 'enum', or 'graph' (default: int). Domain can be specified in parenthesis (defaults: %s)" % ", ".join(["%s(%s)" % (name, d) for name, (type, f, d) in TYPES.iteritems()]),
         ),
         optparse.make_option(
             '--flush', action='store_true', dest='flush',
@@ -120,24 +163,29 @@ class Command(base.BaseCommand):
 
         streams = []
         for i in range(nstreams):
-            if types is None or types[i][0] != 'enum':
-                downsamplers = datastream.backend.value_downsamplers
-            else:
-                downsamplers = []
-
             if types is not None:
-                type = types[i]
+                typ = types[i]
             else:
-                type = ('int', '')
+                typ = ('int', '')
+
+            if typ[0] == 'enum':
+                value_type = 'nominal'
+                downsamplers = ['count']
+            elif typ[0] == 'graph':
+                value_type = 'graph'
+                downsamplers = ['count']
+            else:
+                value_type = 'numeric'
+                downsamplers = datastream.backend.value_downsamplers
 
             visualization_value_downsamplers = []
             for downsampler in ['mean', 'min', 'max']:
                 if downsampler in downsamplers:
                     visualization_value_downsamplers.append(downsampler)
 
-            type_constructor, random_function, default_domain = TYPES[type[0]]
-            domain = type[1] or default_domain
-            domain_range = map(type_constructor, domain.split(','))
+            type_constructor, random_function, default_domain = TYPES[typ[0]]
+            domain = typ[1] or default_domain
+            domain_range = [type_constructor(d) for d in domain.split(',')]
 
             stream_id = datastream.ensure_stream(
                 {'title': 'Stream %d' % i},
@@ -146,8 +194,8 @@ class Command(base.BaseCommand):
                     'unit_description': 'random, domain: %s' % domain,
                     'stream_number': i,
                     'visualization': {
-                        'type': 'state' if type is 'enum' else 'line',
-                        'hidden': False,
+                        'type': 'state' if typ is 'enum' else 'line',
+                        'hidden': True if typ is 'graph' else False,
                         'value_downsamplers': visualization_value_downsamplers,
                         'time_downsamplers': ['mean'],
                         # A special case just for demo purposes. We set just minimum so that
@@ -157,14 +205,15 @@ class Command(base.BaseCommand):
                 },
                 downsamplers,
                 datastream.Granularity.Seconds,
+                value_type=value_type,
             )
 
-            streams.append((stream_id, type))
+            streams.append((stream_id, typ))
 
         span = span.split(' ')
         if len(span) == 1 and span[0]:
             span = span[0]
-            for val, key in (('days', 'd'), ('hours', 'h')):
+            for val, key in (('days', 'd'), ('hours', 'h'), ('minutes', 'm'), ('seconds', 's')):
                 if span[-1] == key:
                     try:
                         s = int(span[:-1])
@@ -186,7 +235,7 @@ class Command(base.BaseCommand):
         elif len(span) == 2:
             try:
                 # TODO: Support also timezone in the datetime format
-                span_from, span_to = map(lambda x: datetime.datetime.strptime(x, '%Y-%m-%dT%H:%M:%S'), span)
+                span_from, span_to = [datetime.datetime.strptime(x, '%Y-%m-%dT%H:%M:%S') for x in span]
             except ValueError:
                 raise base.CommandError("Use time format 'yyyy-mm-ddThh:mm:ss' (i.e. '2007-03-04T21:08:12').")
 
@@ -199,9 +248,9 @@ class Command(base.BaseCommand):
                 self.stdout.write("Appending %d values from %s to %s.\n" % (((td.seconds + td.days * 24 * 3600) // interval * len(streams)), span_from, span_to))
 
             while span_from <= span_to:
-                for stream_id, (type, domain) in streams:
-                    type_constructor, random_function, default_domain = TYPES[type]
-                    value = random_function(*map(type_constructor, (domain or default_domain).split(',')))
+                for stream_id, (typ, domain) in streams:
+                    type_constructor, random_function, default_domain = TYPES[typ]
+                    value = random_function(*[type_constructor(d) for d in (domain or default_domain).split(',')])
                     datastream.append(stream_id, value, span_from)
 
                 span_from += datetime.timedelta(seconds=interval)
@@ -218,9 +267,9 @@ class Command(base.BaseCommand):
             self.stdout.write("Appending real-time value(s) to stream(s) every %s seconds.\n" % interval)
 
         while True:
-            for stream_id, (type, domain) in streams:
-                type_constructor, random_function, default_domain = TYPES[type]
-                value = random_function(*map(type_constructor, (domain or default_domain).split(',')))
+            for stream_id, (typ, domain) in streams:
+                type_constructor, random_function, default_domain = TYPES[typ]
+                value = random_function(*[type_constructor(d) for d in (domain or default_domain).split(',')])
 
                 if verbose > 1:
                     self.stdout.write("Appending value '%s' to stream '%s'.\n" % (value, stream_id))
